// Bender Shader
// 
// Copyright Outerspace Software, all rights reserved

string description = "This effect bends your layer.";

float4x4	World;
float4x4	ViewProjection;
float4x4	WorldViewProjection;
float4x4	WorldInverseTranspose;
float4		LightPos;
float4		EyePos;

float4		AmbientCol;
float4		DiffuseCol;
float4		SpecularCol;
float		Power = 100;

float4		TextureResolution0;

bool optimization=false;

float4 Prop0
<
string UIName="Bending Angle";
float UIMin=-0.04;
float UIMax= 0.04;
float3 UIDefault=float3(0,0,0);
int UISliders=1;
float UIScale=25;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

float4 Prop1
<
string UIName="Bending Direction";
float UIMin=-3.14159;
float UIMax=3.14159;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=57.29577;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

float4 Prop2
<
string UIName="Bending Centre";
float UIMin=-100;
float UIMax=100;
float3 UIDefault=float3(0,31,0);
int UISliders=2;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,31,0,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float2 tex:		TEXCOORD0;
	float4 modelpos:	TEXCOORD1;
	float3 normal:		TEXCOORD2;	
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

float3 Rotate(float3 a, bool inInvert){
	if(Prop1.x==0 && Prop1.y==0 && Prop1.z==0) return a;

	float sa=sin(Prop1.x);
	float ca=cos(Prop1.x);
	float sb=sin(Prop1.y);
	float cb=cos(Prop1.y);
	float sg=sin(Prop1.z);
	float cg=cos(Prop1.z);

	float3 b;

	if(!inInvert){
		b.x=a.x*ca-a.z*sa;
		b.y=a.y;
		b.z=a.x*sa+a.z*ca;

		a.x=b.x;
		a.y=b.y*cb-b.z*sb;	
		a.z=b.y*sb+b.z*cb;

		b.x=a.x*cg-a.y*sg;
		b.y=a.x*sg+a.y*cg;
		b.z=a.z;
	}

	if(inInvert){
		sa=-sa;
		sb=-sb;
		sg=-sg;
	
		b.x=a.x*cg-a.y*sg;
		b.y=a.x*sg+a.y*cg;
		b.z=a.z;

		a.x=b.x;
		a.y=b.y*cb-b.z*sb;	
		a.z=b.y*sb+b.z*cb;

		b.x=a.x*ca-a.z*sa;
		b.y=a.y;
		b.z=a.x*sa+a.z*ca;
	}

	return b;
}

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;      

	float4 p=input.position;
	float3 n=input.normal;

//	if(Prop0.x!=0){
	if(Prop0.x<-0.00001 || Prop0.x>0.00001){
	
		p.xyz=Rotate(p.xyz,false);
		n=Rotate(n,false);

		p.xyz-=Prop2.xyz;

		float alfa = p.y * Prop0.x;
		float r = 1.0/Prop0.x;

		float dx=p.x;

		p.x=-r+cos(alfa)*(r+dx);
		p.y=   sin(alfa)*(r+dx);

		p.xyz+=Prop2.xyz;

		float3 n2=n;
		n2.x=n.x*cos(alfa)-n.y*sin(alfa);
		n2.y=n.x*sin(alfa)+n.y*cos(alfa);
		n=n2;

		p.xyz=Rotate(p.xyz,true);
		n=Rotate(n,true);
	}
	
	float4 Pos=mul(p, World);
	output.position=mul(Pos, ViewProjection);
	
	output.tex = input.tex;
	output.modelpos = Pos;
	output.normal = mul(n,WorldInverseTranspose);
	return output;
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;

	float3 NormalVector		= normalize(input.normal);
	float3 LightVector		= normalize(LightPos - input.modelpos);
	float3 EyeVector		= normalize(EyePos   - input.modelpos);
	float3 LightReflectionVector	= -reflect(LightVector,NormalVector);

	//ambient component
	output.color=AmbientCol;

	//diffuse component
	output.color += DiffuseCol * saturate(dot(NormalVector, LightVector));

	//specular component
	output.color += SpecularCol * pow(saturate(dot(EyeVector, LightReflectionVector)),Power);
	
	output.color.w = DiffuseCol.w;

	if(TextureResolution0.x>0){
		output.color *= tex2D(MyTextureSampler0, input.tex);
	}
	
	return output;
}

technique Shader
{
    pass P0
    {
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}

